-- Test edit_contact
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(17);

set search_path to auth, numerus, public;

select has_function('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]']);
select function_lang_is('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'plpgsql');
select function_returns('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'uuid');
select isnt_definer('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]']);
select volatility_is('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'volatile');
select function_privs_are('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'guest', array []::text[]);
select function_privs_are('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'invoicer', array ['EXECUTE']);
select function_privs_are('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'admin', array ['EXECUTE']);
select function_privs_are('numerus', 'edit_contact', array ['uuid', 'text', 'text', 'email', 'uri', 'tax_details', 'tag_name[]'], 'authenticator', array []::text[]);


set client_min_messages to warning;
truncate contact cascade;
truncate payment_method cascade;
truncate company cascade;
reset client_min_messages;


set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (1, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 111)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (111, 1, 'cash', 'cash')
     , (112, 1, 'bank', 'send money to my bank account')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into contact (contact_id, company_id, slug, name, tags)
values (12, 1, '7ac3ae0e-b0c1-4206-a19b-0be20835edd4', 'Contact 1', '{tag1}')
     , (13, 1, 'b57b980b-247b-4be4-a0b7-03a7819c53ae', 'Contact 2', '{tag2}')
     , (14, 1, '12fd031b-8f4d-4ac1-9dde-7df336dc6d52', 'Contact 3', '{tag3}')
;

insert into contact_tax_details (contact_id, business_name, vatin, address, city, province, postal_code, country_code)
values (12, 'Contact 1 Ltd', 'XX555', '', '', '', '', 'ES')
     , (13, 'Contact 2 Ltd', 'XX666', '', '', '', '', 'ES')
;

insert into contact_phone (contact_id, phone)
values (12, '777-777-777')
     , (13, '888-888-888')
;

insert into contact_email (contact_id, email)
values (12, 'c@c')
     , (13, 'd@d')
;

insert into contact_web (contact_id, uri)
values (12, 'https://1/')
     , (13, 'https://2/')
;


select lives_ok(
	$$ select edit_contact('7ac3ae0e-b0c1-4206-a19b-0be20835edd4', 'Contact 2.1', '999-999-999', 'c1@c1', 'https://c', '(Contact 2.1 Ltd,40404040D,"Fake St., 123",City 2.1,Province 2.1,19486,ES)', array['tag1']) $$,
	'Should be able to edit the first contact'
);

select lives_ok(
	$$ select edit_contact('b57b980b-247b-4be4-a0b7-03a7819c53ae', 'Contact 2.2', null, null, null, null, array['tag1', 'tag3']) $$,
	'Should be able to edit the second contact'
);

select lives_ok(
	$$ select edit_contact('12fd031b-8f4d-4ac1-9dde-7df336dc6d52', 'Contact 2.3', '111-111-111', 'd2@d2', 'https://d', '(Contact 2.3 Ltd,41414141L,"Another Fake St., 123",City 2.2,Province 2.2,17417,ES)', array['tag2']) $$,
	'Should be able to edit the third contact'
);

select bag_eq(
	$$ select company_id, name, tags, created_at from contact $$,
	$$ values (1, 'Contact 2.1', '{tag1}'::tag_name[],      CURRENT_TIMESTAMP)
	        , (1, 'Contact 2.2', '{tag1,tag3}'::tag_name[], CURRENT_TIMESTAMP)
	        , (1, 'Contact 2.3', '{tag2}'::tag_name[], CURRENT_TIMESTAMP)
	$$,
	'Should have updated all contacts'
);

select bag_eq(
	$$ select name, business_name, vatin::text, address, city, province, postal_code, country_code::text from contact join contact_tax_details using (contact_id) $$,
	$$ values ('Contact 2.1', 'Contact 2.1 Ltd', 'ES40404040D', 'Fake St., 123',         'City 2.1', 'Province 2.1', '19486', 'ES')
	        , ('Contact 2.3', 'Contact 2.3 Ltd', 'ES41414141L', 'Another Fake St., 123', 'City 2.2', 'Province 2.2', '17417', 'ES')
	$$,
	'Should have updated all contacts’ tax details'
);

select bag_eq(
	$$ select name, phone::text from contact join contact_phone using (contact_id) $$,
	$$ values ('Contact 2.1', '+34 999 99 99 99')
	        , ('Contact 2.3', '+34 111111111')
	$$,
	'Should have updated all contacts’ phone'
);

select bag_eq(
	$$ select name, email::text from contact join contact_email using (contact_id) $$,
	$$ values ('Contact 2.1', 'c1@c1')
	        , ('Contact 2.3', 'd2@d2')
	$$,
	'Should have updated all contacts’ email'
);

select bag_eq(
	$$ select name, uri::text from contact join contact_web using (contact_id) $$,
	$$ values ('Contact 2.1', 'https://c')
	        , ('Contact 2.3', 'https://d')
	$$,
	'Should have updated all contacts’ web'
);


select *
from finish();

rollback;
